// shared js file for all acap aplications

// array of all acap for menu
var apps = ["CamStreamer", "CamOverlay", "CamScripter"];

//if IE show message
var is_ie = /MSIE|Trident/.test(window.navigator.userAgent);
if(is_ie){
  $('#test_js_working').text('Your browser doesn\'t support modern version of JavaScript. Please use current version of Chrome, Firefox, Safari or Edge.');
  $('#test_js_working').css('display','block');
  $('body > *:not(#test_js_working)').css('display', 'none');
}

// function - log error
function log_error(message,status){
  console.log('Error: ' + message + '(' + status + ')');
}

// function - get param from camera
async function getParameter(paramList) {
  return await new Promise(function(resolve, reject) {
    $.get("/axis-cgi/param.cgi?action=list&group=" + paramList, function(data) {
      var params = {};
      var lines = data.split('\n');
      for (var i = 0; i < lines.length; i++) {
        var pos = lines[i].indexOf('=');
        if (pos != -1) {
          var name = lines[i].substring(0, pos).trim().toLowerCase();
          var value = lines[i].substring(pos + 1).trim();
          if (name.length && value.length) {
            params[name] = value;
          }
        }
      }
      resolve(params);
    });
  });
}

function setParameter(app_name, paramName, value) {
  var promise = new Promise(function(resolve, reject) {
    var dataJson = {};
    dataJson['action'] = 'update';
    dataJson[app_name + '.' + paramName] = value;

    $.ajax({
      method: "POST",
      cache: true,
      url: "/axis-cgi/param.cgi",
      data: dataJson
    }).done(function(msg) {
      try {
        if (msg.trim().toLowerCase() == 'ok')
          resolve();
        else
          reject();
      } catch(err) {
        reject();
      }

    });
  });
  return promise;
}

async function installAcap(appName, app_name, serverUrl, installPage, defaultPage) {

  await handleInstallAndUpdateProcess(true, appName, app_name, serverUrl, installPage, defaultPage);
}

async function prepareUpdate(appName, appNameNice, serverUrl, latestACAPVersion, currentVersion) {
  // Prepare changelog data and insert to modal, then open modal
  let releaseNotesInfoModal = $('#releaseNotesInfo');
  let releaseNotesInfoModalBody = releaseNotesInfoModal.find('.modal-body');
  let releaseNotesInfoModalTitle = releaseNotesInfoModal.find('.modal-title');
  let titleText = 'Release notes for ' + appNameNice + ' version ' + latestACAPVersion;
  let releaseNoteMsgError = $('<div id="getReleaseNotesMsgError" class="alert alert-danger"></div>');
  let releaseNoteMsgInfo = $('<div id="getReleaseNotesMsgInfo" class="alert alert-info"></div>')

  releaseNotesInfoModalTitle.text(titleText);
  releaseNotesInfoModalBody.html('');
  releaseNotesInfoModal.modal('show');

  // Get release notes info and insert to modal
  try {
    var releaseNotesInfoResponse = await getReleaseNotesForLatestACAPVersion(appName, serverUrl, latestACAPVersion, currentVersion);

    if (releaseNotesInfoResponse.code !== 'OK') {
      releaseNoteMsgError.text('Failed to get release notes.');
      releaseNotesInfoModalBody.html(releaseNoteMsgError);
      log_error(releaseNotesInfoResponse.message, releaseNotesInfoResponse.code);
    } else {

      if (releaseNotesInfoResponse.data.length === 0) {
        releaseNoteMsgInfo.text('No release notes for this update.');
        releaseNotesInfoModalBody.html(releaseNoteMsgInfo);
      }
      let noteRowTemplate = '<div class="row"><div class="col-3">' + appNameNice + ' %VERSION%<br><span class="text-nowrap">%DATE%</span></div><div class="col">%CONTENT%</div></div>';

      releaseNotesInfoResponse.data.forEach(function(note) {
        releaseNotesInfoModalBody.append(
          noteRowTemplate
            .replace('%VERSION%', note.version)
            .replace('%DATE%', note.release_date)
            .replace('%CONTENT%', note.content)
        );
      });
    }
  } catch(error) {
    releaseNoteMsgError.text('Failed to get release notes.');
    releaseNotesInfoModalBody.html(releaseNoteMsgError);
    log_error(error.statusText, error.status);
  }

  // After confirm make update and close modal
  $('#proceedUpdate').click(function() {
    getUpdate(appName, appNameNice, serverUrl, installPage, defaultPage);
    releaseNotesInfoModal.modal('hide');
  });
}

async function getReleaseNotesForLatestACAPVersion(appName, serverUrl, latestACAPVersion, currentVersion) {
  return await new Promise(function(resolve, reject) {
    $.get({
      url: serverUrl + '/index.php?p=ACAPPackage&a=getReleaseNotes',
      data: {
        app_name: appName,
        target_version: latestACAPVersion,
        current_version: currentVersion
      },
      success: function(response) {
        resolve(response);
      },
      error: function(response) {
        reject(response);
      }
    });
  });
}

async function handleInstallAndUpdateProcess(isInstallation, appName, appNameNice, serverUrl, installPage, defaultPage) {
  let params = await getParameter("Brand.ProdNbr,Properties.Firmware.Version");
  let dataModel = params["brand.prodnbr"];
  let dataFirmware = params["properties.firmware.version"];

  // 1. Get package info
  try {
    var packageInfoResponse = await getPackageInfo(appName, serverUrl, dataModel, dataFirmware);
    var name = packageInfoResponse.download_url.split("/");
    var fileName = name[(name.length)-3] + "_" + name[(name.length)-2] + "_" + name[(name.length)-1];
  } catch (error) {
    log_error(error);
    throw Error(error);
  }

  // 2. Download file
  try {
    (isInstallation) ? renderInstallProcess(appName, appNameNice, serverUrl) : renderUpdateProcess(appName, appNameNice, serverUrl);
    var downloadFileBlob = await downloadFile(packageInfoResponse.download_url);
  } catch(error) {
    log_error(error);
    renderErrorDownload(appName, appNameNice, serverUrl);
    throw Error(error);
  }

  // 3. Upload to camera and install
  try {
    await installApp(fileName, downloadFileBlob);
    (isInstallation) ? renderSuccessfullInstallation(appName, appNameNice, serverUrl, defaultPage) : renderSuccessfullUpdate(appName, appNameNice, serverUrl, defaultPage);
  } catch(error) {
    log_error(error);
    renderErrorInstallation(appName, appNameNice, serverUrl);
    throw Error(error);
  }

}

async function getUpdate(appName, appNameNice, serverUrl, installPage, defaultPage) {

  await handleInstallAndUpdateProcess(false, appName, app_name, serverUrl, installPage, defaultPage);
}

async function getPackageInfo(appName, serverUrl, cameraModel, cameraFirmware) {
  return await new Promise(function(resolve, reject) {
    $.get({
      url: serverUrl + '/index.php?p=ACAPPackage&a=getPackageInfo',
      data: {
        camera_model: cameraModel,
        camera_firmware: cameraFirmware,
        app_name: appName
      },
      success: function(response) {
        if (response.code === 200) {
          resolve(response);
        } else {
          reject('Something went wrong with getting compatible package for your camera, please try again.');
        }
      },
      error: function() {
        reject('Something went wrong with getting information about compatible application for your camera, please try again.');
      }
    });
  });
}

async function downloadFile(downloadUrl) {
  return await new Promise(function(resolve, reject) {
    let oReq = new XMLHttpRequest();
    oReq.open("GET", downloadUrl, true);
    oReq.responseType = "blob";
    oReq.onload = function (oEvent) {
      if (oReq.status === 200) {
        resolve(oReq.response);
      } else {
        reject("Error - downloading file. (" + oReq.status + ")");
      }
    };
    oReq.addEventListener("error", function() {
      reject("Error - downloading file. (" + oReq.status + ")");
    });
    oReq.send();
  });
}

async function installApp(fileName, blob) {
  return await new Promise(function(resolve,reject) {
    let form = $('#installForm')[0];
    let formData = new FormData(form);
    formData.append("packfil", blob, fileName);
    $.post({
      url: '/axis-cgi/applications/upload.cgi',
      data: formData,
      contentType: false,
      processData: false
    }).done(function() {
      resolve();
    }).fail(function(response, status, error) {
      reject(error);
    });
  });
}

function renderErrorPrepare(appName, appNameNice, serverUrl){
  var content = '<div class="card card-empty text-center" id="download_error_' + appName + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-exclamation-triangle text-danger align-middle"></span></h1>' +
    '<p class="align-middle">Something went wrong with preparing <a class="text" target="_blank" href="' + serverUrl + '/' + appName + '-about">' + appNameNice + '</a></p>' +
    '</blockquote>' +
    '<p>Please contact our support team on <a href="mailto:support@camstreamer.com">support@camstreamer.com</a>.</p>' +
    '<a href="mailto:support@camstreamer.com" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-life-ring mr-2"></span>Contact support</a>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function renderNotSupportedCamera(appName, appNameNice, serverUrl) {
  var content = '<div class="card card-empty text-center" id="download_error_' + appName + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-exclamation-triangle text-danger align-middle"></span></h1>' +
    '<p class="align-middle">Your camera does not support <a class="text" target="_blank" href="' + serverUrl + '/' + appName + '-about">' + appNameNice + '</a></p>' +
    '</blockquote>' +
    '<br><p>Check compatible cameras <a target="_blank" href="' + serverUrl + '/download-app-all-supported-cameras#' + appName + '-download">here</a>.</p>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function renderErrorDownload(appName, appNameNice, serverUrl) {
  var content = '<div class="card card-empty text-center" id="download_error_' + appName + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-exclamation-triangle text-danger align-middle"></span></h1>' +
    '<p class="align-middle">Error in downloading of <a class="text" target="_blank" href="' + serverUrl + '/' + appName + '-about">' + appNameNice + '</a></p>' +
    '</blockquote>' +
    '<p>Please contact our support team on <a href="mailto:support@camstreamer.com">support@camstreamer.com</a>.</p>' +
    '<a href="mailto:support@camstreamer.com" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-life-ring mr-2"></span>Contact support</a>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function renderErrorInstallation(appName, appNameNice, serverUrl) {
  var content = '<div class="card card-empty text-center" id="installation_error_' + appName + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-exclamation-triangle text-danger align-middle"></span></h1>' +
    '<p class="align-middle">Error in installation of <a class="text" target="_blank" href="' + serverUrl + '/' + appName + '-about">' + appNameNice + '</a></p>' +
    '</blockquote>' +
    '<p>Please contact our support team on <a href="mailto:support@camstreamer.com">support@camstreamer.com</a>.</p>' +
    '<a href="mailto:support@camstreamer.com" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-life-ring mr-2"></span>Contact support</a>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function renderUpdateProcess(appName, appNameNice, serverUrl) {
  $("#" + appNameNice + " .badge").text('updating').removeClass('badge-dark').addClass('badge-info'); // Change badge in menu
  $('div.container[role="main"]').html(''); //empty main container
  $('div.container[role="main"]').append('<div id="content_installation" class="mt-4"></div>'); // Fill it with element
  var content_progress = '<div class="card card-empty text-center" id="installation_progress_' + appName + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-cog fa-spin text-primary align-middle"></span></h1>' +
    '<p class="align-middle">Updating <a class="text" target="_blank" href="' + serverUrl + '/' + appName + '-about">' + appNameNice + '</a></p>' +
    '</blockquote>' +
    '<p>Please wait until the end of this update.</p>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content_progress);
}

function renderInstallProcess(appName, appNameNice, serverUrl) {
  $("#" + appNameNice + " .badge").text('installing').removeClass('badge-dark').addClass('badge-info'); // Change badge in menu
  $('div.container[role="main"]').html(''); //empty main container
  $('div.container[role="main"]').append('<div id="content_installation" class="mt-4"></div>'); // Fill it with element
  var content_progress = '<div class="card card-empty text-center" id="installation_progress_' + appName + '">' +
      '<div class="card-body">' +
      '<blockquote class="blockquote mb-0">' +
      '<h1><span class="mr-2 fa fa-cog fa-spin text-primary align-middle"></span></h1>' +
      '<p class="align-middle">Installing <a class="text" target="_blank" href="' + serverUrl + '/' + appName + '-about">' + appNameNice + '</a></p>' +
      '</blockquote>' +
      '<p>Please wait until the end of this installation.</p>' +
      '</div>' +
      '</div>';

  $('#content_installation').html(content_progress);
}

function renderSuccessfullUpdate(appName, appNameNice, serverUrl, defaultPage) {
  $("#" + appNameNice + " .badge").text('updated').removeClass('badge-info').addClass('badge-success'); //change badge status in menu
  var content = '<div class="card card-empty text-center" id="installation_success_' + appName + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-check-circle text-success align-middle"></span></h1>' +
    '<p class="align-middle">Update of <a class="text" target="_blank" href="' + serverUrl + '/' + appName + '-about">' + appNameNice + '</a> was successfull!</p>' +
    '</blockquote>' +
    '<p>Feel free to contact us via help menu if you have any questions.  Enjoy our awesome app!</p>' +
    '<a href="' + '/local/' + appName + '/' + defaultPage + '" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-archive mr-2"></span>Open app</a>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function renderSuccessfullInstallation(appName, appNameNice, serverUrl, defaultPage) {
  $("#" + appNameNice + " .badge").text('installed').removeClass('badge-info').addClass('badge-success'); //change badge status in menu
  var content = '<div class="card card-empty text-center" id="installation_success_' + appName + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><span class="mr-2 fa fa-check-circle text-success align-middle"></span></h1>' +
    '<p class="align-middle">Installation of <a class="text" target="_blank" href="' + serverUrl + '/' + appName + '-about">' + appNameNice + '</a> was successfull!</p>' +
    '</blockquote>' +
    '<p>Feel free to contact us via help menu if you have any questions.  Enjoy our awesome app!</p>' +
    '<a href="' + '/local/' + appName + '/' + defaultPage + '" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-archive mr-2"></span>Open app</a>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);
}

function getAppList() {
  var promise = new Promise(function(resolve, reject) {
    $.get('/axis-cgi/applications/list.cgi').then(function(xml) {
      var result = xmlToJSON.parseXML(xml);
      var apps = [];
      for (var i = 0; i < result.reply[0]["application"].length; i++) {
        apps.push(result.reply[0]["application"][i]["_attr"]);
      }
      resolve(apps);
    }, reject);
  }.bind(this));
  return promise;
}

function renderMenu(app, serverUrl) {
  if(app === '' || app === undefined){
    app = app_name;
  }

  getAppList().then(function(data) {
    var menu = [];
    if (data != null) {
      for (var i = 0; i < data.length; i++) {
        var menuItem = "";
        var ConfigurationPage = "";
        if (data[i]["ConfigurationPage"] === undefined) {
          ConfigurationPage = "";
        } else {
          ConfigurationPage = data[i]["ConfigurationPage"]["_value"];
        }
        var version = String(data[i]["Version"]["_value"]).replace("-", ".");
        var niceNameApp = data[i]["NiceName"]["_value"];
        var niceName = niceNameApp.replace(' App','');

        menuItem += '<a id="' + niceName + '" class="';
        if (niceName === app) {
          menuItem += 'active ';
        }
        menuItem += 'mr-2 nav-link btn-menu" href="/' + ConfigurationPage;
        menuItem += '">' + niceNameApp + '<span class="ml-2 badge badge-dark" style="font-size: 0.6rem;">';
        menuItem += version + '</span></a>';

        menu[niceName] = menuItem;
      }
    }

    var insertHTML = "";
    for (var o = 0; o < apps.length; o++) {
      // Fill not install apps
      if (menu[apps[o]] === undefined) {
        insertHTML += '<a id="' + apps[o] + '" server-url="' + serverUrl + '" class="';
        if (apps[o] === app) {
          insertHTML += 'active ';
        }
        insertHTML += 'mr-2 nav-link btn-menu install_package" href="javascript:void(0);">' + apps[o] + " App" + '<span class="ml-2 badge badge-dark" style="font-size: 0.6rem;">start trial</span></a>';
      } else {
        insertHTML += menu[apps[o]];
      }
    }

    $('#main_menu').html(insertHTML);

    $('.install_package').on('click', function() {
      notInstalledApp($(this).attr('id'), $(this).attr('server-url'));
    });
  });
}

function notInstalledApp(name, serverUrl) {
  var oldUrl = location.pathname;
  oldUrl = oldUrl.split('/');
  location.href = '/local/' + oldUrl[2].toLowerCase() + '/shared/installApp.html?app='+ name + '&serverUrl=' + serverUrl;
}

function findGetParameter(parameterName) {
  var result = null,
    tmp = [];
  location.search
    .substr(1)
    .split("&")
    .forEach(function (item) {
      tmp = item.split("=");
      if (tmp[0] === parameterName) result = decodeURIComponent(tmp[1]);
    });
  return result;
}

function renderInstallApp(app_name_id, app_name, serverUrl, installPage, defaultPage) {
  document.title = app_name;
  renderStyles(app_name_id);

  var content = '<div class="card card-empty text-center" id="install_div_' + app_name_id + '">' +
    '<div class="card-body">' +
    '<blockquote class="blockquote mb-0">' +
    '<h1><img src="logo_CamStreamer.png" width="30" style="filter:invert(100%);" /></h1>' +
    '<p>Install trial version of <a class="text" target="_blank" href="' + serverUrl + '/' + app_name_id + '-about">' + app_name + '</a></p>' +
    '</blockquote>' +
    '<p>Install trial version of our awesome ' + app_name + ' in one click to your camera.</p>' +
    '<button type="button" id="install_button" class="btn btn-sm btn-primary pt-2 pb-2 pl-4 pr-4"><span class="fas fa-download mr-2"></span>Install</button>' +
    '</div>' +
    '</div>';

  $('#content_installation').html(content);

  // After click start installation process
  $('#install_button').click(function() {
    installAcap(app_name_id, app_name, serverUrl, installPage, defaultPage);
  });
}

function renderStyles(app_name){
  app = app_name.toLowerCase();
  $('head').append('<link rel="stylesheet" type="text/css" href="bt_' + app + '.css">');
}

// Replace server url in links
function replaceServerUrl(serverUrl) {
  $('.serverURL').each(function() {
    var replaced = false;
    if (this.getAttribute('href')) {
      this.setAttribute('href', serverUrl + '/' + this.getAttribute('href').split('/').slice(3).join('/'));
      replaced = true;
    }
    if (this.getAttribute('src')) {
      this.setAttribute('src', serverUrl + '/' + this.getAttribute('src').split('/').slice(3).join('/'));
      replaced = true;
    }
    if (this.getAttribute('original')) {
      this.setAttribute('original', serverUrl + '/' + this.getAttribute('original').split('/').slice(3).join('/'));
      replaced = true;
    }

    if (!replaced)
    {
      console.error('replaceServerUrl: undefined item type: ' + this.tagName + ', content: ' + this.textContent);
    }
  });
}